* File..: INDEXING.PRG
* Author: John Kaster
* Date..: 11/19/1987
* Notes.: These routines are an excellent way to change the way a database
*         is organized, since changing the index for a Clipper application
*         is now just a matter of putting in the new index key expression
*         in a text file, deleting the old index, and running the Clipper
*         program again.
*
*         To use, create a file called "INDEXES.KEY" in the same directory
*         your Clipper application will execute from.  Each line represents
*         an index file name and its key expression.  The index file name
*         and the key expression must be separated by at least one space.
*         The index file name goes first, then the index key expression.
*
*         For example, if you had a database for a phone list, you might wish
*         to use Name, Zip, and Area Code as your indices.  In order to do so,
*         just create the file INDEXES.KEY with the following three lines:
*
*                  Name     UPPER(Last+First)
*                  Zip      Zip
*                  AreaCode SUBSTR(Phone_No,2,3)
*
*         be sure to include a carriage return on the last line.
*
*         Then, in order to check if the index files are there and make them
*         active, simply put the expression
*
*                  CHECKNDX('Name','Zip','AreaCode',.T.)
*
*         after USEing the Phone file.
*
*         Refer to my file INDEMO for the exact source code.
*
*         LINKING:
*         Requires the functions LEFT,ALLTRIM, and RIGHT from either TR.LIB or
*         EXTENDDB.PRG to be linked.  Also requires CLIPASC.OBJ to be linked.
*

FUNC GETIKEY
* Syntax:  GETIKEY( <Index File Name> )
* Notes.:  Performs sequential reads through an ASCII text file named
*          "INDEXES.KEY" until it reaches the EOF marker or the first
*          word in the index is matched to the target index file name.
*
*
PARA Target
PRIV CL,Handle && Private variables for Current Line and DOS File Handle
* Assumes that INDEXES.KEY is what is used for the entire index structure
IF ! FILE('INDEXES.KEY')
  @ 0,0 SAY 'The index key file is missing.  Cannot execute program.'
  QUIT
ENDIF
Handle=OPENASC('INDEXES.KEY','r')
CL=' '
DO WHILE ASCRETCD()#-1.AND.UPPER(LEFT(CL,AT(' ',CL)-1))#UPPER(Target)
  CL=LINEIN(Handle)
ENDDO
CloseASC(Handle)
IF UPPER(LEFT(CL,AT(' ',CL)-1))==UPPER(TARGET)
  CL=ALLTRIM(RIGHT(CL,LEN(CL)-AT(' ',CL)))
ELSE
  CL=' '
ENDIF
RETU CL

PROC CHECKNDX
* Syntax:  CheckNDX( <Index File Names>, <Set (Y/N)> )
* Notes.:  This procedure will check for the existence of the index files
*          <Index File Names> and set the index to those index files if they
*          are found and a boolean TRUE is passed as the last parameter.
*
*          If the <Index File Names> index file is not found, the
*          IKEY() function will be invoked to recall the index key expression
*          from the file "INDEXES.KEY."  Refer to function IKEY() for more
*          information.
*
*          For however many indexes there are set, the procedure set up the
*          index key and order.  As in dBASE, the limit of the indexes is 7.
*
PARA IND1,IND2,IND3,IND4,IND5,IND6,IND7,SetIt
PRIV I,J,K
I=PCOUNT()
IF I<1
  RETU
ENDIF
IF I<2
  SetIt=.T.
ENDIF
IF I<8 && Checking if SetIt was passed as the last parameter
  val=STR(I)
  IF TYPE('IND&Val')='L' && If last index field was logical, must be SetIt
    SetIt=IND&Val
    I=I-1
  ELSE
    SetIt=.T.
  ENDIF
ENDIF
FOR J=1 TO I
  val=STR(J,1)
  IND=Ind&Val
  IF ! FILE( IND + IIF('.' $ IND,'','.NTX') )
    @ 0,0 SAY 'Creating the '+Ind+' index file'
    IKEY=GETIKEY(Ind)
    IF ! EMPTY(IKEY)
      INDE ON &IKEY TO &Ind
    ELSE
      @ 0,0 SAY 'Unable to find the '+Ind+' index key expression in INDEXES.KEY'
      inkey(0)
    ENDIF
    @ 0,0
  ENDIF
NEXT
IF SetIt
* Note that if an index file is missing, this will return a run-time error
  DO CASE
  CASE I=7
    SET INDE TO &Ind1,&Ind2,&Ind3,&Ind4,&Ind5,&Ind6,&Ind7
  CASE I=6
    SET INDE TO &Ind1,&Ind2,&Ind3,&Ind4,&Ind5,&Ind6
  CASE I=5
    SET INDE TO &Ind1,&Ind2,&Ind3,&Ind4,&Ind5
  CASE I=4
    SET INDE TO &Ind1,&Ind2,&Ind3,&Ind4
  CASE I=3
    SET INDE TO &Ind1,&Ind2,&Ind3
  CASE I=2
    SET INDE TO &Ind1,&Ind2
  CASE I=1
    SET INDE TO &Ind1
  ENDC
ENDIF
RETU
